/* lib/windows64.p
 *
 * windows kernel calls
 *
 * use this when _progres is a 64 bit executable or when running source
 *
 */

&IF "{&PROCESS-ARCHITECTURE}" = "64" &THEN
  &global-define XINT 		int64
  &global-define LONGINT 	int64
  &global-define PUTLONGINT	PUT-INT64
  &global-define GETLONGINT	GET-INT64
 &ELSE
  &global-define XINT 		integer
  &global-define LONGINT 	long
  &global-define PUTLONGINT	PUT-LONG
  &global-define GETLONGINT	GET-LONG
&ENDIF

session:add-super-procedure( this-procedure ).

return.


procedure CreateProcessA external "kernel32.dll":
    define input  parameter lpApplicationName    as {&LONGINT}.
    define input  parameter lpCommandline        as character.
    define input  parameter lpProcessAttributes  as {&LONGINT}.
    define input  parameter lpThreadAttributes   as {&LONGINT}.
    define input  parameter bInheritHandles      as {&LONGINT}.
    define input  parameter dCreationFlags       as {&LONGINT}.
    define input  parameter lpEnvironment        as {&LONGINT}.
    define input  parameter lpCurrentDirectory   as {&LONGINT}.
    define input  parameter lpStartupInfo        as {&LONGINT}.
    define input  parameter lpProcessInformation as {&LONGINT}.
    define return parameter bResult              as {&LONGINT}.
end procedure.


procedure OpenProcess external "kernel32.dll" :
    define input  parameter desiredAccess as {&LONGINT}.
    define input  parameter inheritFlag   as {&LONGINT}.
    define input  parameter pid           as {&LONGINT}.
    define return parameter procHdl       as {&LONGINT}.
end procedure.


procedure TerminateProcess external "kernel32.dll" :
    define input  parameter hProcess  as {&LONGINT}.
    define input  parameter uExitCode as {&LONGINT}.
    define return parameter retval    as {&LONGINT}.
end procedure.


procedure CloseHandle external "kernel32.dll":
    define input  parameter procHdl  as {&LONGINT}.
    define return parameter statcode as {&LONGINT}.
end.

procedure GetFileType external "kernel32.dll":
  define input  parameter fileHandle as {&LONGINT}.
  define return parameter result     as {&LONGINT}.
end.

procedure GetStdHandle external "kernel32.dll":
  define input  parameter fileHandle as {&LONGINT}.
  define return parameter result     as {&LONGINT}.
end.


procedure spawn:

  define input  parameter CommandLine as character no-undo.
  define input  parameter WorkingDir  as character no-undo.
  define output parameter PID         as {&XINT}   no-undo.

  define variable wShowWindow   as integer no-undo initial 0.
  define variable bResult       as integer no-undo.
  define variable returnValue   as integer no-undo.

  define variable lpStartupInfo        as memptr no-undo.
  define variable lpProcessInformation as memptr no-undo.
  define variable lpWorkingDirectory   as memptr no-undo.

  set-size ( lpStartupInfo )     = 68.

  {&PUTLONGINT} ( lpStartupInfo, 1 )  = 68.
  {&PUTLONGINT} ( lpStartupInfo, 45 ) = 1. 			/* = STARTF_USESHOWWINDOW */

  put-short( lpStartupInfo, 49 ) = wShowWindow.

  set-size( lpProcessInformation )   = 16.

  if WorkingDir <> "" then
    do:
      set-size( lpWorkingDirectory )     = 256.
      put-string( lpWorkingDirectory, 1 ) = WorkingDir.
    end.

  run CreateProcessA(
    0,
    CommandLine,
    0,
    0,
    0,
    0,
    0,
    ( if WorkingDir = "" then 0 else get-pointer-value( lpWorkingDirectory )),
    get-pointer-value( lpStartupInfo ),
    get-pointer-value( lpProcessInformation ),
    output bResult
  ).

  if bResult <> 0 then     /* release kernel-objects hProcess and hThread: */
    do:
      PID = {&GETLONGINT} ( lpProcessInformation, 9 ).
      run CloseHandle ( input {&GETLONGINT} ( lpProcessInformation, 1 ), output returnValue ).
      run CloseHandle ( input {&GETLONGINT} ( lpProcessInformation, 5 ), output returnValue ).
    end.

  set-size( lpStartupInfo )        = 0.
  set-size( lpProcessInformation ) = 0.
  set-size( lpWorkingDirectory )   = 0.

end procedure.


procedure kill:

  define input parameter PID as {&XINT} no-undo.

  define variable processHandle as {&XINT} no-undo.
  define variable returnValue   as {&XINT} no-undo.

  run OpenProcess ( 1, 0, PID, output processHandle ).

  if processHandle <> 0 then
    do:
      run TerminateProcess ( processHandle, 0, output returnValue ).
      run CloseHandle ( processHandle, output returnValue ).
    end.

end.
