/* lib/zippy.p
 *
 * standard user experience measurement - measure time to access a standard set of db data
 *
 * these meta-schema tables exist in every Progress database and there should
 * always be enough data to support the 500 record target
 *
 * these meta-schema tables will almost certainly always be in the -B cache so
 * there should not be any influence from disk io on the timings
 *
 * the usrExpSHM metric is derived from this data and presented as "records per second"
 *
 */

session:add-super-procedure( this-procedure ).

return.


/* do the actual work
 */

define variable estart    as int64 no-undo.
define variable startDate as date  no-undo.     /* the current date when zippy was run */

procedure zippy:

  define output parameter r as integer no-undo.
  define output parameter t as integer no-undo.

  define variable i  as integer no-undo.
  define variable ex as int64 no-undo.

  assign
    r         = 0
    estart    = mtime
    ex        = estart + 50          /* run for at least 50ms	*/
    startDate = today            
    .

  if ex >= 86400000 then             /* don't run if we're within 50 ms of midnight */
    do:
      t = 1.                         /* prevent divide-by-zero in the caller */
      return.
    end.  

  usrexp: 
  do while mtime < ex:

    /* the above check of ex *should* prevent this procedure from running past midnight.
       but we have seen a case where it does not, so now we also check that the date 
       hasn't rolled over, just to be sure we aren't running too long. */

    if startDate < today then leave usrexp.

    i = 0.
    for each _file no-lock:
      assign
        i = i + 1
        r = r + 1
      .
      if mtime > ex then leave usrexp.
      if i >= 100 then leave.
    end.

    i = 0.
    for each _field no-lock:
      assign
        i = i + 1
        r = r + 1
      .
      if mtime > ex then leave usrexp.
      if i >= 100 then leave.
    end.
  
    i = 0.
    for each _index no-lock:
      assign
        i = i + 1
        r = r + 1
      .
      if mtime > ex then leave usrexp.
      if i >= 100 then leave.
    end.

    i = 0.
    for each _index-field no-lock:
      assign
        i = i + 1
        r = r + 1
      .
      if mtime > ex then leave usrexp.
      if i >= 100 then leave.
    end.

    i = 0.
    for each _storageObject no-lock:
      assign
        i = i + 1
        r = r + 1
      .
      if mtime > ex then leave usrexp.
      if i >= 100 then leave.
    end.

  end.

  t = mtime - estart.
  
  if t < 1 then
    do:
      assign
        r = 0
        t = 1      /* if we ran past midnight then t could be negative */
        .
    end.    

  return.

end.
